﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using BexIDE.GUI;
using BexIDE.Themes;
using BexIDE.TIData;
using BexIDE.Tokens;
using BexIDE.Tools;

namespace BexIDE.Forms {
    /// <summary>
    /// The main form of BexIDE.
    /// </summary>
    public partial class MainForm : Form {

        #region Static fields

        private static string BEXIDE_DIRECTORY = Application.StartupPath + "/";
        private static string THEME_DIRECTORY = BEXIDE_DIRECTORY + "themes/";
        private static string TOKEN_FILE = BEXIDE_DIRECTORY + "tokens.xml";
        private static Size TAB_SIZE = new Size(100, 25);

        #endregion Static fields

        #region Private fields

        private StripRenderer renderer;
        private ThemeLoader themeLoader;
        private List<Theme> themes;
        private TokenCollection tokenLibrary;

        #endregion Private fields

        #region Properties
        
        /// <summary>
        /// Gets or sets the current theme.
        /// </summary>
        public Theme Theme {
            get { return renderer.Theme; }
            set { ApplyTheme(value); }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new main form.
        /// </summary>
        public MainForm() {
            Logger.Begin(BEXIDE_DIRECTORY);
            InitializeComponent();
            
            renderer = new StripRenderer(new DefaultTheme());
            mbrMain.RenderMode = ToolStripRenderMode.ManagerRenderMode;
            mbrMain.Renderer = renderer;

            tabMain.ItemSize = new Size(100, 25);

            themeLoader = new ThemeLoader();
            LoadThemes();
            LoadSettings();

            TokenFileParser tfp = new TokenFileParser(TOKEN_FILE);
            tokenLibrary = tfp.ReadTokens();

            // BEGIN DEBUG STUFFS
            Program p2 = new Program(tokenLibrary);
            p2.Parse(new System.IO.StringReader("\"HELLO World!\""));
            p2.Export("C:/Dev/_files/APPVARS_parsed.8xp");
        }

        #endregion ctor()

        #region Methods
        
        /// <summary>
        /// (Re)Loads all themes.
        /// </summary>
        private void LoadThemes() {
            themes = themeLoader.LoadThemes(THEME_DIRECTORY);
            for (int i = 1; i < mnuThemes.DropDownItems.Count; i++)
                mnuThemes.DropDownItems.RemoveAt(i);
            foreach (Theme theme in themes) {
                ToolStripMenuItem mi = new ToolStripMenuItem(theme.Name);
                mi.Click += new System.EventHandler(themeItem_Click);
                mnuThemes.DropDownItems.Add(mi);
            }
        }

        /// <summary>
        /// Removes all checks from the drop-down items in the Theme sub-menu.
        /// </summary>
        private void ResetThemeChecks() {
            foreach (ToolStripMenuItem item in mnuThemes.DropDownItems)
                item.Checked = false;
        }

        /// <summary>
        /// Loads the settings.
        /// </summary>
        private void LoadSettings() {
            string theme = BexIDE.Properties.Settings.Default.ThemeName;
            ApplyTheme(GetTheme(theme));
            SelectTheme(theme);

            WabbitInterface.WabbitLocation = BexIDE.Properties.Settings.Default.WabbitLoc;

            StringCollection recentFiles = Properties.Settings.Default.RecentFiles;
            if (recentFiles == null)
                return;

            foreach (string file in recentFiles) {
                ToolStripMenuItem item = GenerateRecentFileMenuItem(file);
                mnuRecentFiles.DropDownItems.Add(item);
            }
        }

        /// <summary>
        /// Selects the theme in the drop down list whose name is given.
        /// </summary>
        /// <param name="name">The name of the theme to select.</param>
        private void SelectTheme(string name) {
            ResetThemeChecks();
            foreach (ToolStripMenuItem item in mnuThemes.DropDownItems)
                if (item.Text.Equals(name)) {
                    item.Checked = true;
                    break;
                }
        }

        /// <summary>
        /// Gets a theme whose name is applied to.
        /// </summary>
        /// <param name="name">Name of the theme.</param>
        /// <returns>A theme whose name is the same as the provided string.</returns>
        private Theme GetTheme(string name) {
            foreach (Theme theme in themes)
                if (theme.Name.Equals(name))
                    return theme;
            return new DefaultTheme();
        }

        /// <summary>
        /// Applies a theme.
        /// </summary>
        /// <param name="t">The theme to apply.</param>
        private void ApplyTheme(Theme t) {
            renderer = new StripRenderer(t);
            mbrMain.Renderer = renderer;

            tabMain.Theme = t;
            tabMain.Invalidate();
        }
        
        /// <summary>
        /// Generates a menu item for recent files.
        /// </summary>
        /// <param name="file">File to generate item for.</param>
        /// <returns>A menu item ready to be added to the recent files list.</returns>
        private ToolStripMenuItem GenerateRecentFileMenuItem(string file) {
            ToolStripMenuItem item = new ToolStripMenuItem(file);
            // item.Click += (sender, e) => MessageBox.Show(this, ((ToolStripMenuItem)sender).Text);
            return item;
        }

        #endregion Methods

        #region Events

        /// <summary>
        /// Occurs when a theme item is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void themeItem_Click(object sender, EventArgs e) {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;

            ResetThemeChecks();
            item.Checked = true;

            ApplyTheme(GetTheme(item.Text));
        }

        #endregion

    }
}