﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using BexIDE.Themes;

namespace BexIDE.GUI {
    /// <summary>
    /// The MenuStrip, ToolStrip, and StatusStrip renderer for BexIDE.
    /// </summary>
    public class StripRenderer : ToolStripRenderer {

        #region Fields

        private Theme theme;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets or sets the theme of this StripRenderer.
        /// </summary>
        public Theme Theme {
            get { return theme; }
            set { theme = value; }
        }

        #endregion

        #region ctor()

        /// <summary>
        /// Creates a new StripRenderer.
        /// </summary>
        /// <param name="theme">The theme of the renderer.</param>
        public StripRenderer(Theme theme) {
            Theme = theme;
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Draws a custom rectangle.
        /// </summary>
        /// <param name="g">Graphics to draw to.</param>
        /// <param name="r">Rectangle to derive from.</param>
        /// <param name="c">Color to draw rectangle as.</param>
        private void DrawCustomRectangle(Graphics g, Rectangle r, Color c) {
            Pen p = new Pen(c);

            g.DrawLine(p, r.Left + 1, r.Top, r.Right - 1, r.Top);
            g.DrawLine(p, r.Left + 1, r.Bottom, r.Right - 1, r.Bottom);

            g.DrawLine(p, r.Left, r.Top + 1, r.Left, r.Bottom - 1);
            g.DrawLine(p, r.Right, r.Top + 1, r.Right, r.Bottom - 1);
        }

        /// <summary>
        /// Fills a custom rectangle.
        /// </summary>
        /// <param name="g">Graphics to draw to.</param>
        /// <param name="r">Rectangle to derive from.</param>
        /// <param name="c">Color to fill rectangle with.</param>
        private void FillCustomRectangle(Graphics g, Rectangle r, Color c) {
            Brush b = new SolidBrush(c);

            r.X++;
            r.Width--;
            r.Y++;
            r.Height--;

            g.FillRectangle(b, r);
        }

        /// <summary>
        /// Recalculates a rectangle's dimensions.
        /// </summary>
        /// <param name="r">Rectangle to recalculate.</param>
        /// <returns>A recalculated rectangle.</returns>
        private Rectangle RecalculateRectangle(Rectangle r) {
            r.X++;
            r.Width -= 2;
            r.Height--;
            return r;
        }

        #endregion Methods

        #region Overriden methods

        /// <summary>
        /// Initializes the tool strip.
        /// </summary>
        protected override void Initialize(ToolStrip toolStrip) {
            base.Initialize(toolStrip);

            // Set the forecolor and back color of the item, and remove the grip "thing"
            toolStrip.ForeColor = Theme.ForeColor;
            toolStrip.BackColor = Theme.BackColor;
            toolStrip.GripStyle = ToolStripGripStyle.Hidden;
        }

        /// <summary>
        /// Initializes a new item to be added to the tool strip.
        /// </summary>
        protected override void InitializeItem(ToolStripItem item) {
            base.InitializeItem(item);

            // Set forecolor and backcolor
            item.ForeColor = Theme.ForeColor;
            item.BackColor = Theme.BackColor;
        }

        /// <summary>
        /// Renders an arrow that indicates a sub-menu.
        /// </summary>
        protected override void OnRenderArrow(ToolStripArrowRenderEventArgs e) {
            e.ArrowColor = Theme.TextColor;

            base.OnRenderArrow(e);
        }

        /// <summary>
        /// Renders an item's background.
        /// </summary>
        protected override void OnRenderMenuItemBackground(ToolStripItemRenderEventArgs e) {
            base.OnRenderMenuItemBackground(e);

            if (e.Item is ToolStripMenuItem) {
                if (e.Item.IsOnDropDown && e.Item.Selected) {
                    Rectangle rect = RecalculateRectangle(e.Item.ContentRectangle);

                    FillCustomRectangle(e.Graphics, rect, Theme.MenuItemSelectedFill);
                    DrawCustomRectangle(e.Graphics, rect, Theme.MenuItemSelectedBorder);
                }
                else if (!e.Item.IsOnDropDown && e.Item.Selected) {
                    Rectangle rect = e.Item.ContentRectangle;
                    rect.X += 2;
                    rect.Width -= 5;

                    FillCustomRectangle(e.Graphics, rect, Theme.MenuItemSelectedFill);
                    DrawCustomRectangle(e.Graphics, rect, Theme.MenuItemSelectedBorder);
                }
            }
        }

        /// <summary>
        /// Renders a menu separator.
        /// </summary>
        protected override void OnRenderSeparator(ToolStripSeparatorRenderEventArgs e) {
            base.OnRenderSeparator(e);

            ToolStripItem item = e.Item;

            int left = item.Width / 2 - 40;
            int right = left + 80;
            int top = item.Height - 2;
            int bottom = item.Height - 1;

            using (Pen p = new Pen(Theme.SeperatorDark))
                e.Graphics.DrawLine(p, new Point(left, top), new Point(right, top));

            using (Pen p = new Pen(Theme.SeparatorLight))
                e.Graphics.DrawLine(p, new Point(left, bottom), new Point(right, bottom));
        }

        /// <summary>
        /// Renders a tool strip's background.
        /// </summary>
        protected override void OnRenderToolStripBackground(ToolStripRenderEventArgs e) {
            base.OnRenderToolStripBackground(e);

            if (e.ToolStrip is MenuStrip) {
                LinearGradientBrush grad = new LinearGradientBrush(e.AffectedBounds, Theme.MenuGradientPrimary, Theme.MenuGradientSecondary, 90f);
                using (grad)
                    e.Graphics.FillRectangle(grad, e.AffectedBounds);
            }
            else if (e.ToolStrip is ToolStripDropDownMenu) {
                e.Graphics.FillRectangle(new SolidBrush(Theme.DropdownFill), e.AffectedBounds);
            }
        }

        /// <summary>
        /// Renders a tool strip's border.
        /// </summary>
        protected override void OnRenderToolStripBorder(ToolStripRenderEventArgs e) {
            if (e.ToolStrip is ToolStripDropDownMenu) {
                Rectangle rect = e.AffectedBounds;
                rect.Width--;
                rect.Height--;

                DrawCustomRectangle(e.Graphics, rect, Theme.DropdownBorder);
            }
        }

        /// <summary>
        /// Renders text.
        /// </summary>
        protected override void OnRenderItemText(ToolStripItemTextRenderEventArgs e) {
            if (e.Item.Selected)
                e.TextColor = Color.Black;
            else
                e.TextColor = Theme.TextColor;
            
            base.OnRenderItemText(e);
        }

        #endregion Overriden methods

    }
}