﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace BexIDE.TIData {
    public abstract class File8x : IFile8x {

        #region Fields

        private static byte[] header = {
            (byte)'*', (byte)'*', (byte)'T', (byte)'I', (byte)'8', (byte)'3', (byte)'F', (byte)'*',
            0x1A, 0x0A, 0x00
        };

        private char[] comment, name;
        private byte[] data;
        private byte version;
        private TypeID type;
        private ArchiveState memState;
        private string location;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets the comment of the file.
        /// </summary>
        public string Comment {
            get { return new string(comment); }
        }

        /// <summary>
        /// Gets the name of the file.
        /// </summary>
        public string Name {
            get { return new string(name); }
        }

        /// <summary>
        /// Gets the read data of the file.
        /// </summary>
        public byte[] Data {
            get { return data; }
        }

        /// <summary>
        /// Gets the length of the data.
        /// </summary>
        public ushort DataLength {
            get { return (ushort)data.Length; }
        }

        /// <summary>
        /// Gets or sets the version of the file.
        /// </summary>
        public byte Version {
            get { return version; }
            set { version = value; }
        }

        /// <summary>
        /// Gets or sets the type of the file.
        /// </summary>
        public TypeID Type {
            get { return type; }
            set { type = value; }
        }

        /// <summary>
        /// Gets or sets the archive flag of the file.
        /// </summary>
        public ArchiveState MemState {
            get { return memState; }
            set { memState = value; }
        }

        /// <summary>
        /// The location in the file-system of the file.
        /// </summary>
        public string Location {
            get { return location; }
            set { location = value; }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new .8X* file class.
        /// </summary>
        public File8x() {
            comment = new char[42];
            name = new char[8];
            version = 0;
            type = TypeID.AppVar;
            memState = ArchiveState.RAM;
            data = new byte[0];
            location = "";

            SetName("PROGRAM");
            SetComment("Made with BexIDE.");
        }

        /// <summary>
        /// Creates a new .8X* class file from a specified file.
        /// </summary>
        /// <param name="file">The file to load.</param>
        public File8x(string file) : this() {
            Import(file);
            location = file;
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Sets the comment of the file.
        /// </summary>
        /// <param name="c">The new comment.</param>
        public void SetComment(string c) {
            for (int i = 0; i < 42; i++)
                comment[i] = '\0';

            for (int i = 0; i < Math.Min(c.Length, 42); i++)
                comment[i] = c[i];
        }

        /// <summary>
        /// Sets the name of the file.
        /// </summary>
        /// <param name="n">The new name.</param>
        public void SetName(string n) {
            Array.Clear(name, 0, 8);
            n.CopyTo(0, name, 0, Math.Min(n.Length, 8));
        }

        /// <summary>
        /// Sets the data of the file.
        /// </summary>
        /// <param name="d">The new data.</param>
        public void SetData(byte[] d) {
            data = new byte[d.Length];
            d.CopyTo(data, 0);
        }

        /// <summary>
        /// Exports the current file.
        /// </summary>
        /// <param name="file">File to export information to.</param>
        public void Export(string file) {

            using (BinaryWriter w = new BinaryWriter(File.Create(file))) {

                w.Write(header);
                w.Write(comment);
                w.Write((ushort)(DataLength + 19));

                List<byte> buff = new List<byte>();
                buff.AddRange(BitConverter.GetBytes((ushort)0x0D));
                buff.AddRange(BitConverter.GetBytes((ushort)(DataLength + 2)));
                buff.Add((byte)type);
                for (int i = 0; i < 8; i++)
                    buff.Add((byte)name[i]);
                buff.Add(version);
                buff.Add((byte)memState);
                buff.AddRange(BitConverter.GetBytes((ushort)(DataLength + 2)));
                buff.AddRange(BitConverter.GetBytes((ushort)DataLength));
                buff.AddRange(data);

                w.Write(buff.ToArray());
                w.Write(GetChecksum(buff));

            }
        }

        /// <summary>
        /// Imports information from the specified file.
        /// </summary>
        /// <param name="file">File to import.</param>
        public void Import(string file) {
            using (BinaryReader r = new BinaryReader(File.OpenRead(file))) {

                r.ReadBytes(11);
                r.ReadBytes(42).CopyTo(comment, 0);
                ushort plus19 = r.ReadUInt16();
                r.ReadBytes(4);
                type = (TypeID)r.ReadByte();
                r.ReadBytes(8).CopyTo(name, 0);
                version = r.ReadByte();
                memState = (ArchiveState)r.ReadByte();
                ushort plus2 = r.ReadUInt16();
                ushort len = r.ReadUInt16();
                data = r.ReadBytes((int)len);

                Console.WriteLine((plus19 - 19 == len) && (plus2 - 2 == len));

            }
        }

        /// <summary>
        /// Gets the checksum of the file.
        /// </summary>
        public ushort GetChecksum(IEnumerable<byte> buff) {
            ulong tot = 0;
            foreach (byte b in buff)
                tot += (ulong)b;
            return (ushort)(tot % (long)65536);
        }

        #endregion Methods
        
    }
}