﻿using System;
using System.Drawing;

namespace BexIDE.TIData {
    public class Picture : File8x {

        #region Fields

        private byte picNum;
        private PicSize size;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets or sets the number of the picture (zero based).
        /// </summary>
        public byte PicNumber {
            get { return picNum; }
            set {
                picNum = value;
                string name = new string(new char[] { (char)VarPrefix.Image, (char)picNum });
                SetName(name);
            }
        }

        /// <summary>
        /// Gets or sets the size of the picture.
        /// </summary>
        public PicSize Size {
            get { return size; }
            set {
                size = value;
                byte[] data = Data;
                Array.Resize<byte>(ref data, size == PicSize.Default ? 768 : 842);
                base.SetData(data);
            }
        }

        /// <summary>
        /// Gets the width of the picture.
        /// </summary>
        public int Width {
            get { return size == PicSize.Default ? 95 : 96; }
        }

        /// <summary>
        /// Gets the height of the picture.
        /// </summary>
        public int Height {
            get { return size == PicSize.Default ? 63 : 64; }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new picture.
        /// </summary>
        public Picture() : base() {
            picNum = 0;
            size = PicSize.Default;
            Type = TypeID.Image;
        }

        /// <summary>
        /// Creates a new picture from a file.
        /// </summary>
        /// <param name="file">File to load picture data from.</param>
        public Picture(string file) : base(file) {
            picNum = 0;
            size = PicSize.Modded;
            Type = TypeID.Image;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Converts this TI-84 picture into a bitmap.
        /// </summary>
        public Bitmap ToBitmap(int scale) {
            Bitmap bmp = new Bitmap(96 * scale, 64 * scale);
            Graphics gfx = Graphics.FromImage(bmp);
            gfx.Clear(Color.White);
            int x = 0, y = 0;
            for (int i = 0; i < DataLength; i++) {
                for (int j = 7; j >= 0; j--) {
                    Rectangle fill = new Rectangle(x, y, scale, scale);
                    if (((Data[i] >> j) & 1) == 1)
                        gfx.FillRectangle(Brushes.Black, fill);
                    else
                        gfx.FillRectangle(Brushes.White, fill);
                    
                    x += scale;
                    if (x == 96 * scale) {
                        x = 0;
                        y += scale;
                    }
                }
            }
            gfx.Flush();
            gfx.Dispose();
            return bmp;
        }

        /// <summary>
        /// Loads data from a bitmap.
        /// </summary>
        /// <param name="file">Bitmap file to load</param>
        public void LoadBitmap(string file) {
            Bitmap bmp = new Bitmap(file);
            byte[] data = new byte[842];
            int width = Math.Min(bmp.Width, 96);
            int height = Math.Min(bmp.Height, 64);
            string bin = "";
            int i = 0;

            for (int y = 0; y < height; y++) {
                for (int x = 0; x < width; x++) {
                    if (IsBlack(bmp.GetPixel(x, y)))
                        bin += "1";
                    else
                        bin += "0";

                    if (bin.Length == 8) {
                        byte b = Convert.ToByte(bin, 2);
                        data[i] = b;
                        bin = "";
                        i++;
                    }
                }
            }

            base.SetData(data);
        }

        private bool IsBlack(Color c) {
            return ((c.R == 0) && (c.B == 0) && (c.G == 0));
        }

        #endregion Methods

    }

    public enum PicSize {
        Default,
        Modded
    }
}