﻿using BexIDE.Tokens;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace BexIDE.TIData {
    public class Program : File8x {

        #region Fields

        private TokenCollection tokenLib;
        private List<Token> parsedTokens;
        
        #endregion Fields

        #region Properties
        
        /// <summary>
        /// Gets or sets the token library used for parsing.
        /// </summary>
        public TokenCollection TokenLibrary {
            get { return tokenLib; }
            set { tokenLib = value; }
        }
        
        /// <summary>
        /// Gets or sets the parsed token library.
        /// </summary>
        public List<Token> ParsedTokens {
            get { return parsedTokens; }
            set {
                parsedTokens = value;
                SetBaseData();
            }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new program.
        /// </summary>
        /// <param name="tokens">Token collection to use for parsing.</param>
        public Program(TokenCollection tokens) : base() {
            tokenLib = tokens;
            parsedTokens = new List<Token>();
            Type = TypeID.Program;
        }

        /// <summary>
        /// Creates a new program.
        /// </summary>
        /// <param name="tokens">Token collection to use for parsing.</param>
        /// <param name="file">File to initially load data from.</param>
        public Program(TokenCollection tokens, string file) : base(file) {
            tokenLib = tokens;
            parsedTokens = new List<Token>();
            Type = TypeID.Program;
            ParseBaseData();
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Protects the program from being edited on-calc.
        /// </summary>
        public void Protect() {
            Type = TypeID.ProtProgram;
        }

        /// <summary>
        /// Unprotects the program so it is not editable on-calc.
        /// </summary>
        public void Unprotect() {
            Type = TypeID.Program;
        }

        /// <summary>
        /// Gets the concatenation of the text of all parsed tokens.
        /// </summary>
        /// <returns></returns>
        public string GetText() {
            string text = "";
            foreach (Token t in parsedTokens) {
                text += t.Text;
            }
            return text;
        }

        /// <summary>
        /// Gets all parsed tokens.
        /// </summary>
        public Token[] GetParsedTokens() {
            Token[] tkns = new Token[parsedTokens.Count];
            parsedTokens.CopyTo(tkns, 0);
            return tkns;
        }

        /// <summary>
        /// Parses a text stream.
        /// </summary>
        /// <param name="reader">Stream to parse.</param>
        public void Parse(TextReader reader) {
            StringBuilder buff = new StringBuilder();
            parsedTokens.Clear();
            while (reader.Peek() != -1) {
                buff.Append((char)reader.Read());
                List<Token> matches = tokenLib.FindMatching(buff.ToString());
                if (matches.Count == 1) {
                    parsedTokens.Add(matches[0]);
                    int diff = matches[0].Text.Length - buff.Length;
                    while (diff > 0) {
                        reader.Read();
                        diff--;
                    }
                    buff.Length = 0;
                }
                if (matches.Count == 0 && buff.Length != 0 && buff.ToString() != "") {
                    char last = buff.ToString()[buff.Length - 1];
                    buff.Remove(buff.Length - 1, 1);
                    Token t = tokenLib[buff.ToString()];
                    if (t != null)
                        parsedTokens.Add(t);
                    buff.Length = 1;
                    buff[0] = last;
                }
            }

            if (buff.Length != 0) {
                Token t = tokenLib[buff.ToString()];
                if (t != null)
                    parsedTokens.Add(t);
            }

            SetBaseData();
        }

        /// <summary>
        /// Sets the base data.
        /// </summary>
        private void SetBaseData() {
            byte[] newData = new byte[tokenLib.ByteCount(parsedTokens)];
            int i = 0;
            foreach (Token t in parsedTokens) {
                foreach (byte b in t.Bytes) {
                    newData[i] = b;
                    i++;
                }
            }
            base.SetData(newData);
        }

        /// <summary>
        /// Parses base data into tokens.
        /// </summary>
        private void ParseBaseData() {
            parsedTokens.Clear();
            List<byte> buff = new List<byte>();
            for (int i = 0; i < DataLength; i++) {
                buff.Add(Data[i]);
                Token t = tokenLib[buff.ToArray()];
                if (t != null) {
                    parsedTokens.Add(t);
                    buff.Clear();
                }
            }
        }

        #endregion Methods

    }
}