﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Xml;

namespace BexIDE.Themes {
    /// <summary>
    /// Provides an easy way to load themes.
    /// </summary>
    public class ThemeLoader {

        // As with the token loader, please do not judge me on this file alone.
        // I took a nearly two month break from this and did not go back to
        // look at all of the code.

        #region Fields

        private List<Theme> _themes;

        #endregion Fields

        #region ctor()

        /// <summary>
        /// Creates a new ThemeLoader.
        /// </summary>
        public ThemeLoader()
        {
            _themes = new List<Theme>();
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Gets the list of themes.
        /// </summary>
        /// <returns>List of themes.</returns>
        public List<Theme> GetThemes()
        {
            return _themes;
        }

        /// <summary>
        /// Loads themes from the specified directory.
        /// </summary>
        /// <param name="dir">Directory to load themes from.</param>
        /// <returns>A list of all loaded themes.</returns>
        public List<Theme> LoadThemes(string dir)
        {
            _themes.Clear();                                                                    //Clear previously loaded themes

            foreach (string file in Directory.GetFiles(dir, "*.xml"))                              //For each XML file in the supplied directory
            {
                using (XmlTextReader r = new XmlTextReader(file))
                {
                    Theme t = new Theme();

                    while (r.Read())                                                            //While we can read
                    {
                        if (r.NodeType == XmlNodeType.Element)                                  //If we're at an element
                        {
                            string elemName = r.Name;                                           //Get its name
                            switch (r.Name)                                                     //Switch through cases of the name
                            {
                                case "theme":                                                   //<theme>
                                    string name = r.GetAttribute("name");                       //Get the name of the theme
                                    if (string.IsNullOrEmpty(name))                             //If the name is non-existent, throw an error.
                                        throw new Exception(string.Format("Theme must have a name. ({0})", file));
                                    t.Name = name;                                              //Otherwise set the name
                                    break;
                                default:                                                        //Anything else...
                                    r.Read();                                                   //Consume the element
                                    if (r.NodeType != XmlNodeType.Text)                         //Make sure we're reading text
                                        throw new Exception(string.Format("Expected value for {0}. ({1})", r.Name, file));

                                    string val = r.Value;                                       //Get the value
                                    if (string.IsNullOrEmpty(val))                              //Again, it must exist
                                        throw new Exception(string.Format("Value for {0} cannot be empty. ({1})", r.Name, file));

                                    // Get the color
                                    Color c;
                                    if (val.StartsWith("#"))
                                        c = ColorTranslator.FromHtml(val);
                                    else
                                        c = Color.FromName(val);

                                    // Cycle through properties
                                    switch (elemName)
                                    {
                                        case "forecolor":
                                            t.ForeColor = c;
                                            break;
                                        case "backcolor":
                                            t.BackColor = c;
                                            break;
                                        case "textcolor":
                                            t.TextColor = c;
                                            break;
                                        case "sepdark":
                                            t.SeperatorDark = c;
                                            break;
                                        case "seplight":
                                            t.SeparatorLight = c;
                                            break;
                                        case "menugradprim":
                                            t.MenuGradientPrimary = c;
                                            break;
                                        case "menugradsec":
                                            t.MenuGradientSecondary = c;
                                            break;
                                        case "selborder":
                                            t.MenuItemSelectedBorder = c;
                                            break;
                                        case "selfill":
                                            t.MenuItemSelectedFill = c;
                                            break;
                                        case "dropborder":
                                            t.DropdownBorder = c;
                                            break;
                                        case "dropfill":
                                            t.DropdownFill = c;
                                            break;
                                    }
                                    break;
                            }
                        }
                        else if (r.NodeType == XmlNodeType.EndElement && r.Name.Equals("theme"))//If we're done reading a token
                        {
                            _themes.Add(t);                                                     //Add the theme
                            t = new Theme();                                                    //(Allow for more than one theme per file)
                        }
                    }
                }
            }

            return _themes;
        }

        #endregion

    }
}