﻿using System;
using System.Drawing;

namespace BexIDE.Tokens {
    /// <summary>
    /// Provides information to help easily keep track of TI-84+/SE program file tokens.
    /// </summary>
    public class Token {

        #region Fields

        private string text;
        private byte[] bytes;
        private string group;
        private string syntax;
        private string desc;
        private Color color;
        private FontStyle fontStyle;
        
        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets or sets the textual representation of the token.
        /// </summary>
        public string Text {
            get { return text; }
            set { text = value; }
        }

        /// <summary>
        /// Gets or sets the bytes of the token.
        /// </summary>
        public byte[] Bytes {
            get { return bytes; }
            set { bytes = value; }
        }

        /// <summary>
        /// Gets or sets the group this token belongs to.
        /// </summary>
        public string Group {
            get { return group; }
            set { group = value; }
        }

        /// <summary>
        /// Gets or sets the syntax of this token.
        /// </summary>
        public string Syntax {
            get { return syntax; }
            set { syntax = value; }
        }

        /// <summary>
        /// Gets or sets the description of this token.
        /// </summary>
        public string Description {
            get { return desc; }
            set { desc = value; }
        }

        /// <summary>
        /// Gets or sets the color the token should be displayed as.
        /// </summary>
        public Color TokenColor {
            get { return color; }
            set { color = value; }
        }

        /// <summary>
        /// Gets or sets the font style of the token.
        /// </summary>
        public FontStyle TokenStyle {
            get { return fontStyle; }
            set { fontStyle = value; }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new token.
        /// </summary>
        public Token() {
            Text = "";
            Bytes = new byte[1];
            Group = "";
            Syntax = "";
            Description = "";
            TokenColor = Color.Black;
            TokenStyle = FontStyle.Regular;
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Parses a string of byte text (i.e. $01) and stores the result into Bytes.
        /// </summary>
        /// <param name="byteString">The byte-string to parse.</param>
        public void ParseStringForBytes(string byteString) {
            if (byteString.StartsWith("$"))
                byteString = byteString.Remove(0, 1);

            if (byteString.Length % 2 == 1 || byteString.Length == 0)                           //If the byte string has an invalid length,
                throw new Exception("Invalid byte-string length. Must be non-zero and divisible by 2.");//Throw an error

            Bytes = new byte[byteString.Length / 2];                                            //Create the new byte array

            for (int i = 0; i < byteString.Length; i += 2) {                                    //For each mini-byte-string in the string
                string _byte = byteString.Substring(i, 2);                                      //Get the mini-byte-string
                byte b = Convert.ToByte(_byte, 16);                                             //Convert it to a byte
                Bytes[i / 2] = b;                                                               //Set the byte
            }
        }

        /// <summary>
        /// Returns this token's text.
        /// </summary>
        /// <returns>This token's text.</returns>
        public override string ToString() {
            return Text;
        }

        #endregion Methods

    }
}