﻿using System.Collections.Generic;

namespace BexIDE.Tokens {
    /// <summary>
    /// Provides an easy way to keep track of multiple Tokens.
    /// </summary>
    public class TokenCollection {

        #region Fields

        private Dictionary<string, Token> tokens;
        private Dictionary<byte[], Token> bytes;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets or sets the dictionary of tokens in the collection.
        /// </summary>
        public Dictionary<string, Token> Tokens {
            get { return tokens; }
            set { tokens = value; }
        }

        /// <summary>
        /// Gets the specified token based on the provided text.
        /// </summary>
        /// <param name="text">Text of the token to get.</param>
        public Token this[string text] {
            get {
                if (string.IsNullOrEmpty(text) || FindMatching(text).Count == 0)
                    return null;
                return tokens[text];
            }
        }

        /// <summary>
        /// Gets a token based on provided byte data.
        /// </summary>
        /// <param name="bytes">Byte data of the token to get.</param>
        public Token this[byte[] bytes] {
            get {
                foreach (Token t in tokens.Values)
                    if (AreEqual(t.Bytes, bytes))
                        return t;
                return null;
            }
        }

        #endregion Properties

        #region ctor()

        /// <summary>
        /// Creates a new token collection.
        /// </summary>
        public TokenCollection() {
            tokens = new Dictionary<string, Token>();
            bytes = new Dictionary<byte[], Token>();
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Adds the specified token to the collection.
        /// </summary>
        /// <param name="t">Token to add.</param>
        public void AddToken(Token t) {
            tokens.Add(t.Text, t);
        }

        /// <summary>
        /// Finds the token with the specified text and removes it.
        /// </summary>
        /// <param name="t">Text of the token to remove.</param>
        public void RemoveToken(string t) {
            if (tokens.ContainsKey(t))
                tokens.Remove(t);
        }

        /// <summary>
        /// Returns the token with the specified text.
        /// </summary>
        /// <param name="text">Text of the token to find.</param>
        /// <returns>The token with the specified text.</returns>
        public Token GetToken(string text) {
            return tokens[text];
        }

        /// <summary>
        /// Finds all tokens whose prefixes match the given prefix.
        /// </summary>
        /// <param name="prefix">Prefix to search for.</param>
        public List<Token> FindMatching(string prefix) {
            List<Token> toks = new List<Token>();
            foreach (Token t in tokens.Values)
                if (t.Text.StartsWith(prefix, false, System.Globalization.CultureInfo.CurrentCulture))
                    toks.Add(t);
            return toks;
        }

        /// <summary>
        /// Returns a dictionary containing all unique groups, and all tokens within those groups.
        /// </summary>
        /// <returns>A dictionary containing unique groups with their tokens.</returns>
        public Dictionary<string, List<Token>> GetGroups() {
            Dictionary<string, List<Token>> groups = new Dictionary<string, List<Token>>();                                 //Our dictionary for groups and their tokens
            foreach (Token tkn in tokens.Values) {                                                //For each token
                string tknGroup = tkn.Group;                                                    //Get its group
                if (groups.ContainsKey(tknGroup))                                               //If this group has been "documented,"
                    groups[tknGroup].Add(tkn);                                                  //Just add the token to its list
                else {                                                                          //If the group is "undocumented,"
                    List<Token> list = new List<Token>();                                               //Create a new list
                    list.Add(tkn);                                                              //Add the current token to said list
                    groups.Add(tknGroup, list);                                                 //Add an entry to the dictionary
                }
            }

            return groups;
        }

        /// <summary>
        /// Gets the byte count in a list of tokens.
        /// </summary>
        public int ByteCount(IEnumerable<Token> tokens) {
            int tot = 0;
            foreach (Token t in tokens)
                tot += t.Bytes.Length;
            return tot;
        }

        private bool AreEqual(byte[] first, byte[] second) {
            if (first.Length != second.Length) return false;
            for (int i = 0; i < first.Length; i++)
                if (first[i] != second[i])
                    return false;
            return true;
        }

        #endregion Methods

    }
}