﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Xml;

namespace BexIDE.Tokens {
    /// <summary>
    /// Provides an easy way to parse a token file.
    /// </summary>
    public class TokenFileParser {

        // Please don't judge me by this code...
        // I realize it works, but I haven't gone over it after taking a
        // nearly two month break from this project...

        #region Fields
        
        private XmlTextReader _reader;
        private TokenCollection _tokens;
        private string _file;

        #endregion Fields

        #region ctor()

        /// <summary>
        /// Creates a new token file parser.
        /// </summary>
        /// <param name="file">The file to parse.</param>
        public TokenFileParser(string file)
        {
            _file = file;
        }

        #endregion ctor()

        #region Methods

        /// <summary>
        /// Reads the tokens from the file provided at initialization.
        /// </summary>
        /// <returns>A token collection representing all read tokens.</returns>
        public TokenCollection ReadTokens()
        {
            _tokens = new TokenCollection();                                                    //Clear/Initialize the token collection
            _reader = new XmlTextReader(_file);                                                 //Reset/Initialize the XML reader
            Token currToken = null;                                                             //Current token in-memory

            while (_reader.Read())                                                              //While we can read the XML file
            {
                if (_reader.NodeType == XmlNodeType.Element)                                    //If the node type is an element
                {
                    switch (_reader.Name)                                                       //Go through all element cases
                    {
                        case "token":                                                           //If the element is <token ...>
                            ParseTokenAttributeInfo(ref currToken);                             //Parse it for attribute information
                            break;

                        case "text":                                                            //If the element is <text>
                            _reader.Read();                                                     //Skip the element
                            if (_reader.NodeType == XmlNodeType.Text) {                         //If the current item is text
                                currToken.Text = _reader.Value;                                 //Set the text
                                currToken.Text = currToken.Text.Replace("\\r\\n", "\r\n");
                                if (currToken.Text == "[space]")
                                    currToken.Text = " ";
                            }
                            else                                                                //Else
                                currToken.Text = "<no_text>";                                   //No text
                            break;

                        case "bytes":                                                           //If the element is <bytes>
                            _reader.Read();                                                     //Skip the element
                            if (_reader.NodeType == XmlNodeType.Text)                           //If the element is text
                            {
                                try
                                {
                                    currToken.ParseStringForBytes(_reader.Value);               //Try to parse bytes
                                }
                                catch (Exception ex)
                                {
                                    ThrowError(ex.Message);
                                }
                            }
                            else                                                                //Otherwise
                            {
                                ThrowError("Byte value cannot be empty.");                      //Throw an error
                            }
                            break;

                        case "group":                                                           //If the element is <group>
                            _reader.Read();                                                     //Skip the element
                            if (_reader.NodeType == XmlNodeType.Text)                           //If the element is text
                                currToken.Group = _reader.Value;                                //Set the group
                            else                                                                //Else
                                currToken.Group = "<Default>";                                  //Set the default group
                            break;

                        case "syntax":                                                          //If the element is <syntax>
                            _reader.Read();                                                     //Skip the element
                            if (_reader.NodeType == XmlNodeType.Text)                           //If the element is text
                                currToken.Syntax = _reader.Value;                               //Gets the syntax
                            break;

                        case "desc":                                                            //If the element is <desc>
                            _reader.Read();                                                     //Skip the element
                            if (_reader.NodeType == XmlNodeType.Text)                           //If the element is text
                                currToken.Description = _reader.Value;                          //Get the description
                            break;

                        default:                                                                //Unrecognized element!
                            Debug.WriteLine("Unrecognized XML element: " + _reader.Name);
                            break;
                    }
                }

                // If we've arrived at the end of a token definition and the current token is not null
                if (_reader.NodeType == XmlNodeType.EndElement && _reader.Name.Equals("token") && currToken != null)
                {
                    _tokens.AddToken(currToken);                                                //Add the token
                    currToken = null;                                                           //Reset the current token
                }
            }

            _reader.Close();                                                                    //Don't forget to close the file!
            return _tokens;
        }

        /// <summary>
        /// Parses a token element from the XML reader.
        /// </summary>
        /// <param name="t">Token to write to.</param>
        private void ParseTokenAttributeInfo(ref Token t)
        {
            t = new Token();
            ParseColorInfo(ref t);
            ParseStyleInfo(ref t);

            while (_reader.NodeType != XmlNodeType.EndElement && !_reader.Name.Equals("token"))
            {
                _reader.Read();
                Debug.WriteLine(_reader.Name);
            }
        }

        /// <summary>
        /// Parses the color information for the token.
        /// </summary>
        /// <param name="t">The token that parsed color information should be applied to.</param>
        private void ParseColorInfo(ref Token t)
        {
            string colorVal = _reader.GetAttribute("color");
            if (string.IsNullOrEmpty(colorVal))
                t.TokenColor = Color.Black;
            else if (colorVal.StartsWith("#"))
                t.TokenColor = ColorTranslator.FromHtml(colorVal);
            else
                t.TokenColor = Color.FromName(colorVal);
        }

        /// <summary>
        /// Parses style information for the token.
        /// </summary>
        /// <param name="t">The token that parsed style information should be applied to.</param>
        private void ParseStyleInfo(ref Token t)
        {
            string styleVal = _reader.GetAttribute("style");
            string[] styles = styleVal.Split('|');
            FontStyle style = FontStyle.Regular;

            foreach (string s in styles) {
                switch (s.ToUpper()) {
                    case "REGULAR":
                        style |= FontStyle.Regular;
                        break;
                    case "ITALIC":
                        style |= FontStyle.Italic;
                        break;
                    case "BOLD":
                        style |= FontStyle.Bold;
                        break;
                    case "UNERLINE":
                        style |= FontStyle.Underline;
                        break;
                    case "STRIKEOUT":
                        style |= FontStyle.Strikeout;
                        break;
                }
            }

            t.TokenStyle = style;
        }

        /// <summary>
        /// Throws an error message.
        /// </summary>
        /// <param name="msg">Error message.</param>
        private void ThrowError(string msg) {
            throw new Exception(string.Format("{0}:{1} {2}", _reader.LineNumber, _reader.LinePosition, msg));
        }

        #endregion Methods

    }
}